!=======================================================================
!                   _____________________________
!               \\//-----------------------------
!                )(   //=  //= //= //// //)) //
!               //\\ //_  //  //  /__/ //\\ //
!=======================================================================
!                   XEFFORT LIBRARY SAMPLES
!=======================================================================
! FuncDlg.f90 - Module for handling the dialog containing graph
! formula and parameters
!
! 2005. Xeffort.com
!     Jugoslav Dujic (jdujic@uns.ns.ac.yu)
! 
! You are free to use the code for both commercial and non-commercial
! use. The code is provided as-is without warranties. You are free to
! distribute and modify this source code provided that the list of 
! original author(s) remains untouched and your revisions are indicated
! as such. Contributions are welcome.
!=======================================================================

MODULE FuncDlg

USE XEFFORT
USE View

IMPLICIT NONE

TYPE(X_WINDOW)::           xFuncDlg
!========================================================================
CONTAINS
!=======================================================================
!Creates the dialog for entering properties
SUBROUTINE XFuncDlg_Create()

LOGICAL::         bSt
INTEGER::         iSt, hKey, i
CHARACTER(256)::  sItem

INCLUDE "resource.fd"

bSt = XLoadDialog(IDD_FUNCTION, xFuncDlg)

bSt = XModelessDialog(xFuncDlg, SW_SHOW)
bSt = XCtlSetSub(xFuncDlg, IDOK, xFuncDlg_OnIdok)
bSt = XCtlSetSub(XFuncDlg, IDC_LOCK, XFuncDlg_OnLock)

!Read the MRU list of functions from the registry
hKey = XRegOpen(HKEY_CURRENT_USER, "Software\XFT\XGraph\1.0", iStatus = XREG_OLD)
IF (hKey.NE.0) THEN
   DO i=0,9
      iSt = XRegRead(hKey, "Formula"//XString(i), sItem)
      IF (iSt.NE.0) EXIT
      iSt = XCtlSet(xFuncDlg, IDC_FORMULA, sItem, 0)
   END DO
   CALL XRegClose(hKey)
END IF

END SUBROUTINE XFuncDlg_Create 
!=======================================================================
!Callback for dialog's "Evaluate!" button. Runs the parser and
!evaluates the function in 500 points.
SUBROUTINE xFuncDlg_OnIdok(xWnd, idCmd, nCode)

USE interpreter

INCLUDE 'resource.fd'
 
TYPE(X_WINDOW)::  xWnd
INTEGER, INTENT(IN):: idCmd, nCode

CHARACTER(256)::  sFormula, sLFormula, sItem
INTEGER::         i, iError, nItems, iSt
LOGICAL::         bSt, bFound
REAL::            x, dx
CHARACTER(10)::   sTemp
TYPE(X_MENU)::    xMenu

m_iStatus = E_OK

bSt = XCtlGet(xWnd, IDC_FORMULA, sFormula)
bSt = XCtlGet(xWnd, IDC_X0, sTemp)
READ(sTemp, *, IOSTAT=iError) xGr%xMin
IF (iError.NE.0) THEN
   m_sError = "Invalid lower bound."
   m_iStatus = E_ERROR
END IF
bSt = XCtlGet(xWnd, IDC_X1, sTemp)
READ(sTemp, *, IOSTAT=iError) xGr%xMax
IF (iError.NE.0) THEN
   m_sError = "Invalid upper bound."
   m_iStatus = E_ERROR
END IF
IF (xGr%xMax < xGr%xMin) THEN
   m_sError = "Upper bound must be greater than the lower."
   m_iStatus = E_ERROR
END IF

IF (m_iStatus .EQ. E_OK) THEN
   m_sError=""
   xGr%yMin = +HUGE(0.)
   xGr%yMax = -HUGE(0.)
   DO i = 1,500
      x = xGr%xMin + i*(xGr%xMax-xGr%xMin)/500
      xGr%wXY(i)%fx = x
      sTemp = ''
      xGr%wXY(i)%fy = f_evaluatefn(sFormula, "x", (/dble(x)/), m_sError)
      IF (m_sError.NE.'OK') THEN
         m_iStatus = E_ERROR
         EXIT
      ELSE
         m_iStatus = E_OK
      END IF
      xGr%yMin = MIN(xGr%yMIn, xGr%wXY(i)%fy)
      xGr%yMax = MAX(xGr%yMax, xGr%wXY(i)%fy)
   END DO
   !Add small offset at top and bottom
   xGr%yMin = xGr%yMin - 0.05*ABS(xGr%yMin)
   xGr%yMax = xGr%yMax + 0.05*ABS(xGr%yMax)


   !Pushes the formula text onto "stack" of combo box:
   bSt = XCtlGet(xWnd, IDC_FORMULA, nItems, CTL_NUMITEMS)
   sLFormula = TOLOWER(sFormula)
   bFound = .FALSE.
   DO i=1,nItems
      bSt = XCtlGet(xWnd, IDC_FORMULA, sItem, i)
      IF (TOLOWER(sItem).EQ.sFormula) THEN
         !Move the current formula to top of the stack
         iSt = XCtlSendMessage(xWnd, IDC_FORMULA, CB_DELETESTRING, i-1, 0)
         sFormula = TRIM(sFormula)//CHAR(0)
         iSt = XCtlSendMessage(xWnd, IDC_FORMULA, CB_INSERTSTRING, 0, LOC(sFormula))
         bFound = .TRUE.
         EXIT
      END IF
   END DO
   IF (.NOT.bFound) THEN
      sFormula = TRIM(sFormula)//CHAR(0)
      bSt = XCtlSendMessage(xWnd, IDC_FORMULA, CB_INSERTSTRING, 0, LOC(sFormula))
      IF (nItems.GT.10) THEN
         bSt = XCtlSendMessage(xWnd, IDC_FORMULA, CB_DELETESTRING, 10, 0)
      END IF
   END IF
   bSt = XCtlSet(xWnd, IDC_FORMULA, sFormula)
END IF

xMenu = XGetWindowMenu(XW_FRAME)
bSt = XMenuSet(xMenu, IDM_FILE_SAVEAS, m_iStatus.EQ.E_OK, XC_ENABLE)
bSt = XMenuSet(xMenu, IDM_COPY_WMF, m_iStatus.EQ.E_OK, XC_ENABLE)
bSt = XMenuSet(xMenu, IDM_COPY_BMP, m_iStatus.EQ.E_OK, XC_ENABLE)

!Force redrawing of view window.
CALL XUpdateWindow(xView)

END SUBROUTINE xFuncDlg_OnIdok
!=======================================================================
!Callback for "Lock size" checkbox. Locks the image size (sets the
!X_GRAPH.bLocked property).
SUBROUTINE XFuncDlg_OnLock(xWnd, idCmd, nCode)
 
TYPE(X_WINDOW)::  xWnd
INTEGER, INTENT(IN):: idCmd, nCode

LOGICAL::   bSt
INTEGER::   iX, iY

bSt = XCtlGet(xWnd, idCmd, xGr%bLocked)
!Set the fixed image size.
bSt = XGetWindowPos(xView, iX, iY, xGr%iWidth, xGr%iHeight)
IF (xGr%bLocked) THEN
   !Set hscroll and vscroll styles
   bSt = XSetWindowStyle(xView, WS_HSCROLL+WS_VSCROLL)
ELSE
   !Remove the scrollbars.
   bSt = XScrollbarSet(xView, SB_VERT, iMin=0, iMax=1, iPage=2)
   bSt = XScrollbarSet(xView, SB_HORZ, iMin=0, iMax=1, iPage=2)
END IF
CALL XUpdateWindow(xView)

END SUBROUTINE XFuncDlg_OnLock
!========================================================================
!Store the MRU list of functions into the registry
SUBROUTINE XFuncDlg_SaveRegistry()

INCLUDE "resource.fd"

INTEGER::         hKey, i
LOGICAL::         bSt
CHARACTER(256)::  sItem

hKey = XRegOpen(HKEY_CURRENT_USER, "Software\XFT\XGraph\1.0")
IF (hKey.NE.0) THEN
   DO i=0,9
      bSt = XCtlGet(xFuncDlg, IDC_FORMULA, sItem, i+1)
      IF (.NOT.bSt) EXIT
      bSt = XRegWrite(hKey, "Formula"//XString(i), TRIM(sItem))
   END DO
   CALL XRegClose(hKey)
END IF

END SUBROUTINE XFuncDlg_SaveRegistry
!========================================================================
END MODULE FuncDlg